@@ -2,7 +2,7 @@ import os
 import localstack_client.config
 
 # LocalStack version
-VERSION = '0.11.1.1'
+VERSION = '0.11.3.1'
 
 # constant to represent the "local" region, i.e., local machine
 REGION_LOCAL = 'local'
@@ -16,25 +16,9 @@ HEADER_LOCALSTACK_TARGET = 'x-localstack-target'
 
 # backend service ports, for services that are behind a proxy (counting down from 4566)
 DEFAULT_PORT_EDGE = 4566
-DEFAULT_PORT_APIGATEWAY_BACKEND = 4565
-DEFAULT_PORT_KINESIS_BACKEND = 4564
-DEFAULT_PORT_DYNAMODB_BACKEND = 4563
-DEFAULT_PORT_S3_BACKEND = 4562
-DEFAULT_PORT_SNS_BACKEND = 4561
-DEFAULT_PORT_SQS_BACKEND = 4560
-DEFAULT_PORT_ELASTICSEARCH_BACKEND = 4559
-DEFAULT_PORT_CLOUDFORMATION_BACKEND = 4558
-DEFAULT_PORT_STEPFUNCTIONS_BACKEND = 4557
-DEFAULT_PORT_IAM_BACKEND = 4556
-DEFAULT_PORT_EC2_BACKEND = 4555
-DEFAULT_PORT_KMS_BACKEND = 4554
-DEFAULT_PORT_EVENTS_BACKEND = 4553
-DEFAULT_PORT_LOGS_BACKEND = 4552
-DEFAULT_PORT_SSM_BACKEND = 4551
-DEFAULT_PORT_STS_BACKEND = 4550
-DEFAULT_PORT_SECRETSMANAGER_BACKEND = 4549
 DEFAULT_PORT_WEB_UI = 8080
 
+# host name for localhost
 LOCALHOST = 'localhost'
 
 # version of the Maven dependency with Java utility code
@@ -65,7 +49,7 @@ PATH_USER_REQUEST = '_user_request_'
 
 # name of LocalStack Docker image
 DOCKER_IMAGE_NAME = 'localstack/localstack'
-DOCKER_IMAGE_NAME_LIGHT = 'localstack/localstack-light'
+DOCKER_IMAGE_NAME_FULL = 'localstack/localstack-full'
 
 # backdoor API path used to retrieve or update config variables
 CONFIG_UPDATE_PATH = '/?_config_'
@@ -116,7 +100,10 @@ LOCALSTACK_WEB_PROCESS = 'LOCALSTACK_WEB_PROCESS'
 LOCALSTACK_INFRA_PROCESS = 'LOCALSTACK_INFRA_PROCESS'
 
 # Hardcoded AWS account ID used by moto
-MOTO_ACCOUNT_ID = '123456789012'
+MOTO_ACCOUNT_ID = TEST_AWS_ACCOUNT_ID
+
+# default AWS region us-east-1
+AWS_REGION_US_EAST_1 = 'us-east-1'
 
 # Default lambda registry
 DEFAULT_LAMBDA_CONTAINER_REGISTRY = 'lambci/lambda'
