@@ -5,10 +5,12 @@ import json
 import logging
 from requests.models import Response
 from localstack import config
+from localstack.services import plugins
 from localstack.constants import HEADER_LOCALSTACK_TARGET, HEADER_LOCALSTACK_EDGE_URL, LOCALSTACK_ROOT_FOLDER
-from localstack.utils.common import run, is_root, TMP_THREADS
+from localstack.utils.common import run, is_root, TMP_THREADS, to_bytes, truncate, to_str, get_service_protocol
 from localstack.utils.common import safe_requests as requests
-from localstack.services.generic_proxy import ProxyListener, GenericProxy
+from localstack.services.generic_proxy import ProxyListener, start_proxy_server
+from localstack.services.sqs.sqs_listener import is_sqs_queue_url
 
 LOG = logging.getLogger(__name__)
 
@@ -23,6 +25,9 @@ class ProxyListenerEdge(ProxyListener):
         if method == 'OPTIONS':
             return 200
 
+        if path.split('?')[0] == '/health':
+            return serve_health_endpoint(method, path, data)
+
         # kill the process if we receive this header
         headers.get(HEADER_KILL_SIGNAL) and os._exit(0)
 
@@ -32,39 +37,41 @@ class ProxyListenerEdge(ProxyListener):
         headers[HEADER_LOCALSTACK_EDGE_URL] = 'https://%s' % host
 
         # extract API details
-        _, port, path, host = get_api_from_headers(headers, path)
+        api, port, path, host = get_api_from_headers(headers, path)
+
+        if port and int(port) < 0:
+            return 404
 
         if not port:
-            # detect S3 presigned URLs
-            if 'AWSAccessKeyId=' in path or 'Signature=' in path:
-                port = config.PORT_S3
-            # assume that this is an S3 GET request with URL path `/<bucket>/<key ...>`
-            # TODO: move S3 public URLs to a separate port/endpoint, OR check ACLs here first
-            if method == 'GET' and '/' in path.strip('/'):
-                port = config.PORT_S3
+            port = get_port_from_custom_rules(method, path, data, headers) or port
 
         if not port:
-            LOG.info('Unable to find forwarding rule for host "%s", path "%s", target header "%s", auth header "%s"' %
-                     (host, path, target, auth_header))
+            if api in ['', None, '_unknown_']:
+                truncated = truncate(data)
+                LOG.info(('Unable to find forwarding rule for host "%s", path "%s", '
+                    'target header "%s", auth header "%s", data "%s"') % (host, path, target, auth_header, truncated))
+            else:
+                LOG.info(('Unable to determine forwarding port for API "%s" - please '
+                    'make sure this API is enabled via the SERVICES configuration') % api)
             response = Response()
             response.status_code = 404
             response._content = '{"status": "running"}'
             return response
 
-        use_ssl = config.USE_SSL
-
         connect_host = '%s:%s' % (config.HOSTNAME, port)
-        url = 'http%s://%s%s' % ('s' if use_ssl else '', connect_host, path)
+        url = '%s://%s%s' % (get_service_protocol(), connect_host, path)
         headers['Host'] = host
         function = getattr(requests, method.lower())
         if isinstance(data, dict):
             data = json.dumps(data)
 
-        response = function(url, data=data, headers=headers, verify=False)
+        response = function(url, data=data, headers=headers, verify=False, stream=True)
         return response
 
 
 def get_api_from_headers(headers, path=None):
+    """ Determine API and backend port based on Authorization headers. """
+
     target = headers.get('x-amz-target', '')
     host = headers.get('host', '')
     auth_header = headers.get('authorization', '')
@@ -78,10 +85,12 @@ def get_api_from_headers(headers, path=None):
     try:
         credential_scope = auth_header.split(',')[0].split()[1]
         _, _, _, service, _ = credential_scope.split('/')
-        result = service, config.service_port(service)
+        result = service, get_service_port_for_account(service, headers)
     except Exception:
         pass
 
+    result_before = result
+
     # Fallback rules and route customizations applied below
 
     if host.endswith('cloudfront.net'):
@@ -96,14 +105,80 @@ def get_api_from_headers(headers, path=None):
         result = 's3', config.PORT_S3
     elif result[0] == 'states' in auth_header or host.startswith('states.'):
         result = 'stepfunctions', config.PORT_STEPFUNCTIONS
+    elif result[0] == 'monitoring':
+        result = 'cloudwatch', config.PORT_CLOUDWATCH
     elif '.execute-api.' in host:
         result = 'apigateway', config.PORT_APIGATEWAY
     elif target.startswith('DynamoDBStreams') or host.startswith('streams.dynamodb.'):
-        result = 'dynamodbstreams', config.PORT_DYNAMODBSTREAMS
+        # Note: DDB streams requests use ../dynamodb/.. auth header, hence we also need to update result_before
+        result = result_before = 'dynamodbstreams', config.PORT_DYNAMODBSTREAMS
     elif ls_target == 'web' or path == '/graph':
         result = 'web', config.PORT_WEB_UI
 
-    return result[0], result[1], path, host
+    return result[0], result_before[1] or result[1], path, host
+
+
+def is_s3_form_data(data_bytes):
+    if(to_bytes('key=') in data_bytes):
+        return True
+    if(to_bytes('Content-Disposition: form-data') in data_bytes and to_bytes('name="key"') in data_bytes):
+        return True
+    return False
+
+
+def serve_health_endpoint(method, path, data):
+    if method == 'GET':
+        reload = 'reload' in path
+        return plugins.get_services_health(reload=reload)
+    if method == 'PUT':
+        data = json.loads(to_str(data))
+        plugins.set_services_health(data)
+        return {'status': 'OK'}
+
+
+def get_port_from_custom_rules(method, path, data, headers):
+    """ Determine backend port based on custom rules. """
+
+    # detect S3 presigned URLs
+    if 'AWSAccessKeyId=' in path or 'Signature=' in path:
+        return config.PORT_S3
+
+    # heuristic for SQS queue URLs
+    if is_sqs_queue_url(path):
+        return config.PORT_SQS
+
+    # DynamoDB shell URLs
+    if path.startswith('/shell') or path.startswith('/dynamodb/shell'):
+        return config.PORT_DYNAMODB
+
+    data_bytes = to_bytes(data or '')
+
+    if path == '/' and to_bytes('QueueName=') in data_bytes:
+        return config.PORT_SQS
+
+    # TODO: move S3 public URLs to a separate port/endpoint, OR check ACLs here first
+    stripped = path.strip('/')
+    if method in ['GET', 'HEAD'] and '/' in stripped:
+        # assume that this is an S3 GET request with URL path `/<bucket>/<key ...>`
+        return config.PORT_S3
+
+    # detect S3 URLs
+    if stripped and '/' not in stripped:
+        if method == 'PUT':
+            # assume that this is an S3 PUT bucket request with URL path `/<bucket>`
+            return config.PORT_S3
+        if method == 'POST' and is_s3_form_data(data_bytes):
+            # assume that this is an S3 POST request with form parameters or multipart form in the body
+            return config.PORT_S3
+
+    # detect S3 requests sent from aws-cli using --no-sign-request option
+    if 'aws-cli/' in headers.get('User-Agent', ''):
+        return config.PORT_S3
+
+
+def get_service_port_for_account(service, headers):
+    # assume we're only using a single account, hence return the static port mapping from config.py
+    return config.service_port(service)
 
 
 def do_start_edge(port, use_ssl, asynchronous=False):
@@ -117,8 +192,7 @@ def do_start_edge(port, use_ssl, asynchronous=False):
     # get port and start Edge
     print('Starting edge router (http%s port %s)...' % ('s' if use_ssl else '', port))
     # use use=True here because our proxy allows both, HTTP and HTTPS traffic
-    proxy = GenericProxy(port, ssl=True, update_listener=ProxyListenerEdge())
-    proxy.start()
+    proxy = start_proxy_server(port, use_ssl=True, update_listener=ProxyListenerEdge())
     if not asynchronous:
         proxy.join()
     return proxy
