@@ -1,6 +1,8 @@
 import json
 import uuid
 import logging
+
+from json import JSONDecodeError
 from localstack.utils.aws import aws_stack
 
 LOG = logging.getLogger(__name__)
@@ -11,7 +13,12 @@ def sqs_error_to_dead_letter_queue(queue_arn, event, error):
     queue_url = aws_stack.get_sqs_queue_url(queue_arn)
     attrs = client.get_queue_attributes(QueueUrl=queue_url, AttributeNames=['RedrivePolicy'])
     attrs = attrs.get('Attributes', {})
-    policy = json.loads(attrs.get('RedrivePolicy') or '{}')
+    try:
+        policy = json.loads(attrs.get('RedrivePolicy') or '{}')
+    except JSONDecodeError:
+        LOG.warning('Parsing RedrivePolicy {} failed, Queue: {}'.format(attrs.get('RedrivePolicy'), queue_arn))
+        return
+
     target_arn = policy.get('deadLetterTargetArn')
     if not target_arn:
         return
@@ -48,7 +55,10 @@ def _send_to_dead_letter_queue(source_type, source_arn, dlq_arn, event, error):
     if ':sqs:' in dlq_arn:
         queue_url = aws_stack.get_sqs_queue_url(dlq_arn)
         sqs_client = aws_stack.connect_to_service('sqs')
-        sqs_client.send_message(QueueUrl=queue_url, MessageBody=message, MessageAttributes=message_attrs)
+        result = sqs_client.send_message(QueueUrl=queue_url, MessageBody=message, MessageAttributes=message_attrs)
+        result_code = result.get('ResponseMetadata', {}).get('HTTPStatusCode')
+        if not result_code or result_code >= 400:
+            LOG.info('Unable to send message to dead letter queue %s: %s' % (queue_url, result_code))
     elif ':sns:' in dlq_arn:
         sns_client = aws_stack.connect_to_service('sns')
         sns_client.publish(TopicArn=dlq_arn, Message=message, MessageAttributes=message_attrs)
